@@ -663,17 +663,21 @@ class GeoArray(object):
 
         self.__dict__ = state
 
-    def calc_mask_nodata(self, fromBand=None, overwrite=False):
-        """Calculates a no data mask with (values: 0=nodata; 1=data)
+    def calc_mask_nodata(self, fromBand=None, overwrite=False, flag='all'):
+        # type: (int, bool, str) -> np.ndarray
+        """Calculates a no data mask with values False (=nodata) and True (=data).
 
-        NOTE:   Only pixel containing the nodata values in ALL bands are recognized as nodata pixel. If they contain a
-                pixel value different from the nodata value in any band they are good data pixels.
-
-        :param fromBand:  <int> index of the band to be used (if None, all bands are used)
-        :param overwrite: <bool> whether to overwrite existing nodata mask that has already been calculated
+        :param fromBand:   index of the band to be used (if None, all bands are used)
+        :param overwrite:  whether to overwrite existing nodata mask that has already been calculated
+        :param flag:       algorithm how to flag pixels (default: 'all')
+                           'all': flag those pixels as nodata that contain the nodata value in ALL bands
+                           'any': flag those pixels as nodata that contain the nodata value in ANY band
         :return:
         """
         if self._mask_nodata is None or overwrite:
+            if flag not in ['all', 'any']:
+                raise ValueError(flag)
+
             assert self.ndim in [2, 3], "Only 2D or 3D arrays are supported. Got a %sD array." % self.ndim
             arr = self[:, :, fromBand] if self.ndim == 3 and fromBand is not None else self[:]
 
@@ -689,21 +693,42 @@ class GeoArray(object):
                 elif arr.ndim == 2:
                     mask = ~np.isnan(arr)
                 else:
-                    idx_1st_databand = np.argwhere(~nanbands)[0][0]
-                    mask = ~np.isnan(arr[:, :, idx_1st_databand])
-                    mask[~mask] = np.any(~np.isnan(arr[~mask]), axis=1)
+                    arr_1st_databand = arr[:, :, np.argwhere(~nanbands)[0][0]]
+                    arr_remain = arr[:, :, ~nanbands][:, :, 1:]
+
+                    mask = ~np.isnan(arr_1st_databand)  # True where 1st data band has data
+
+                    if flag == 'all':
+                        # ALL bands need to contain np.nan to flag the mask as nodata
+                        # overwrite the mask at nodata positions (False) with True in case there is data in ANY band
+                        mask[~mask] = np.any(~np.isnan(arr_remain[~mask]), axis=1)
+                    else:
+                        # ANY band needs to contain np.nan to flag the mask as nodata
+                        # overwrite the mask at data positions (True) with False in case there is np.nan in ANY band
+                        mask[mask] = ~np.any(np.isnan(arr_remain[mask]), axis=1)
 
             else:
                 bandmeans = np.mean(np.mean(arr, axis=0), axis=0)
+                nodatabands = bandmeans == self.nodata
 
                 if np.nanmean(bandmeans) == self.nodata:
                     mask = np.full(arr.shape[:2], False)
                 elif arr.ndim == 2:
                     mask = arr != self.nodata
                 else:
-                    idx_1st_databand = np.argwhere(bandmeans != self.nodata)[0][0]
-                    mask = np.array(arr[:, :, idx_1st_databand] != self.nodata)
-                    mask[~mask] = np.any(arr[~mask] != self.nodata, axis=1)
+                    arr_1st_databand = arr[:, :, np.argwhere(~nodatabands)[0][0]]
+                    arr_remain = arr[:, :, ~nodatabands][:, :, 1:]
+
+                    mask = np.array(arr_1st_databand != self.nodata)  # True where 1st data band has data
+
+                    if flag == 'all':
+                        # ALL bands need to contain nodata to flag the mask as such
+                        # overwrite the mask at nodata positions (False) with True in case there is data in ANY band
+                        mask[~mask] = np.any(arr_remain[~mask] != self.nodata, axis=1)
+                    else:
+                        # ANY band needs to contain nodata to flag the mask as such
+                        # overwrite the mask at data positions (True) with False in case there is nodata in ANY band
+                        mask[mask] = ~np.any(arr_remain[mask] == self.nodata, axis=1)
 
             self.mask_nodata = mask
 
@@ -1105,7 +1130,7 @@ class GeoArray(object):
             # sample image down / normalize
             xdim, ydim = \
                 (self.columns * res_factor, self.rows * res_factor) if res_factor else \
-                tuple((np.array([self.columns, self.rows]) / (np.array([self.columns, self.rows]).max() / 1000)))
+                tuple(np.array([self.columns, self.rows]) / (max([self.columns, self.rows]) / 1000))
             xdim, ydim = int(xdim), int(ydim)
 
         if xdim or ydim or out_prj:
@@ -1205,7 +1230,8 @@ class GeoArray(object):
             cS, cE = xlim if isinstance(xlim, (tuple, list)) else (0, self.columns - 1)
             rS, rE = ylim if isinstance(ylim, (tuple, list)) else (0, self.rows - 1)
 
-            image2plot = np.array(rescale_intensity(image2plot, in_range=(vmin, vmax)))
+            # noinspection PyTypeChecker
+            image2plot = rescale_intensity(image2plot, in_range=(vmin, vmax))  # type: np.ndarray
 
             def get_hv_image(b):
                 # FIXME ylabels have the wrong order
@@ -1297,7 +1323,8 @@ class GeoArray(object):
                   origin='upper', transform=crs_in, extent=list(self.box.boundsMap))
 
         # show grid lines
-        # NOTE: the current version of cartopy can only label PlateCarree grid lines - (crs=crs) is not yet supported
+        # NOTE: The current version of cartopy can only label PlateCarree grid lines - (crs=crs_out) is not yet
+        #       supported and the labels are just missing in that case.
         ax.gridlines(draw_labels=True, linewidth=2, color='gray', alpha=0.5, linestyle='--')
 
         if return_map:
